import datetime as dt
from typing import Any, List, TypeVar
from pathlib import Path

import numpy as np
import numpy.typing as npt

_SCT = TypeVar("_SCT", bound=np.generic, covariant=True)

class SubClass(np.ndarray[Any, np.dtype[_SCT]]): ...

subclass: SubClass[np.float64]

AR_f8: npt.NDArray[np.float64]
AR_i8: npt.NDArray[np.int64]
AR_u1: npt.NDArray[np.uint8]
AR_m: npt.NDArray[np.timedelta64]
AR_M: npt.NDArray[np.datetime64]

AR_LIKE_f: List[float]
AR_LIKE_i: List[int]

m: np.timedelta64
M: np.datetime64

b_f8 = np.broadcast(AR_f8)
b_i8_f8_f8 = np.broadcast(AR_i8, AR_f8, AR_f8)

nditer_obj: np.nditer

date_scalar: dt.date
date_seq: list[dt.date]
timedelta_seq: list[dt.timedelta]

def func(a: int) -> bool: ...

reveal_type(next(b_f8))  # E: tuple[Any]
reveal_type(b_f8.reset())  # E: None
reveal_type(b_f8.index)  # E: int
reveal_type(b_f8.iters)  # E: tuple[flatiter[Any]]
reveal_type(b_f8.nd)  # E: int
reveal_type(b_f8.ndim)  # E: int
reveal_type(b_f8.numiter)  # E: int
reveal_type(b_f8.shape)  # E: tuple[builtins.int]
reveal_type(b_f8.size)  # E: int

reveal_type(next(b_i8_f8_f8))  # E: tuple[Any]
reveal_type(b_i8_f8_f8.reset())  # E: None
reveal_type(b_i8_f8_f8.index)  # E: int
reveal_type(b_i8_f8_f8.iters)  # E: tuple[flatiter[Any]]
reveal_type(b_i8_f8_f8.nd)  # E: int
reveal_type(b_i8_f8_f8.ndim)  # E: int
reveal_type(b_i8_f8_f8.numiter)  # E: int
reveal_type(b_i8_f8_f8.shape)  # E: tuple[builtins.int]
reveal_type(b_i8_f8_f8.size)  # E: int

reveal_type(np.inner(AR_f8, AR_i8))  # E: Any

reveal_type(np.where([True, True, False]))  # E: tuple[ndarray[Any, dtype[{intp}]]]
reveal_type(np.where([True, True, False], 1, 0))  # E: ndarray[Any, dtype[Any]]

reveal_type(np.lexsort([0, 1, 2]))  # E: Any

reveal_type(np.can_cast(np.dtype("i8"), int))  # E: bool
reveal_type(np.can_cast(AR_f8, "f8"))  # E: bool
reveal_type(np.can_cast(AR_f8, np.complex128, casting="unsafe"))  # E: bool

reveal_type(np.min_scalar_type([1]))  # E: dtype[Any]
reveal_type(np.min_scalar_type(AR_f8))  # E: dtype[Any]

reveal_type(np.result_type(int, [1]))  # E: dtype[Any]
reveal_type(np.result_type(AR_f8, AR_u1))  # E: dtype[Any]
reveal_type(np.result_type(AR_f8, np.complex128))  # E: dtype[Any]

reveal_type(np.dot(AR_LIKE_f, AR_i8))  # E: Any
reveal_type(np.dot(AR_u1, 1))  # E: Any
reveal_type(np.dot(1.5j, 1))  # E: Any
reveal_type(np.dot(AR_u1, 1, out=AR_f8))  # E: ndarray[Any, dtype[{float64}]]

reveal_type(np.vdot(AR_LIKE_f, AR_i8))  # E: floating[Any]
reveal_type(np.vdot(AR_u1, 1))  # E: signedinteger[Any]
reveal_type(np.vdot(1.5j, 1))  # E: complexfloating[Any, Any]

reveal_type(np.bincount(AR_i8))  # E: ndarray[Any, dtype[{intp}]]

reveal_type(np.copyto(AR_f8, [1., 1.5, 1.6]))  # E: None

reveal_type(np.putmask(AR_f8, [True, True, False], 1.5))  # E: None

reveal_type(np.packbits(AR_i8))  # ndarray[Any, dtype[{uint8}]]
reveal_type(np.packbits(AR_u1))  # ndarray[Any, dtype[{uint8}]]

reveal_type(np.unpackbits(AR_u1))  # ndarray[Any, dtype[{uint8}]]

reveal_type(np.shares_memory(1, 2))  # E: bool
reveal_type(np.shares_memory(AR_f8, AR_f8, max_work=1))  # E: bool

reveal_type(np.may_share_memory(1, 2))  # E: bool
reveal_type(np.may_share_memory(AR_f8, AR_f8, max_work=1))  # E: bool

reveal_type(np.geterrobj())  # E: list[Any]

reveal_type(np.seterrobj([8192, 521, None]))  # E: None

reveal_type(np.promote_types(np.int32, np.int64))  # E: dtype[Any]
reveal_type(np.promote_types("f4", float))  # E: dtype[Any]

reveal_type(np.frompyfunc(func, 1, 1, identity=None))  # ufunc

reveal_type(np.datetime_data("m8[D]"))  # E: Tuple[builtins.str, builtins.int]
reveal_type(np.datetime_data(np.datetime64))  # E: Tuple[builtins.str, builtins.int]
reveal_type(np.datetime_data(np.dtype(np.timedelta64)))  # E: Tuple[builtins.str, builtins.int]

reveal_type(np.busday_count("2011-01", "2011-02"))  # E: {int_}
reveal_type(np.busday_count(["2011-01"], "2011-02"))  # E: ndarray[Any, dtype[{int_}]]
reveal_type(np.busday_count(["2011-01"], date_scalar))  # E: ndarray[Any, dtype[{int_}]]

reveal_type(np.busday_offset(M, m))  # E: datetime64
reveal_type(np.busday_offset(date_scalar, m))  # E: datetime64
reveal_type(np.busday_offset(M, 5))  # E: datetime64
reveal_type(np.busday_offset(AR_M, m))  # E: ndarray[Any, dtype[datetime64]]
reveal_type(np.busday_offset(M, timedelta_seq))  # E: ndarray[Any, dtype[datetime64]]
reveal_type(np.busday_offset("2011-01", "2011-02", roll="forward"))  # E: datetime64
reveal_type(np.busday_offset(["2011-01"], "2011-02", roll="forward"))  # E: ndarray[Any, dtype[datetime64]]

reveal_type(np.is_busday("2012"))  # E: bool_
reveal_type(np.is_busday(date_scalar))  # E: bool_
reveal_type(np.is_busday(["2012"]))  # E: ndarray[Any, dtype[bool_]]

reveal_type(np.datetime_as_string(M))  # E: str_
reveal_type(np.datetime_as_string(AR_M))  # E: ndarray[Any, dtype[str_]]

reveal_type(np.busdaycalendar(holidays=date_seq))  # E: busdaycalendar
reveal_type(np.busdaycalendar(holidays=[M]))  # E: busdaycalendar

reveal_type(np.compare_chararrays("a", "b", "!=", rstrip=False))  # E: ndarray[Any, dtype[bool_]]
reveal_type(np.compare_chararrays(b"a", b"a", "==", True))  # E: ndarray[Any, dtype[bool_]]

reveal_type(np.add_docstring(func, "test"))  # E: None

reveal_type(np.nested_iters([AR_i8, AR_i8], [[0], [1]], flags=["c_index"]))  # E: tuple[nditer]
reveal_type(np.nested_iters([AR_i8, AR_i8], [[0], [1]], op_flags=[["readonly", "readonly"]]))  # E: tuple[nditer]
reveal_type(np.nested_iters([AR_i8, AR_i8], [[0], [1]], op_dtypes=np.int_))  # E: tuple[nditer]
reveal_type(np.nested_iters([AR_i8, AR_i8], [[0], [1]], order="C", casting="no"))  # E: tuple[nditer]



import os
import sys
import ast
import types
import warnings
import unittest
import contextlib
from typing import (
    Literal as L,
    Any,
    AnyStr,
    Callable,
    ClassVar,
    Dict,
    Iterable,
    List,
    NoReturn,
    overload,
    Pattern,
    Sequence,
    Set,
    Tuple,
    Type,
    type_check_only,
    TypeVar,
    Union,
    Final,
    SupportsIndex,
)

from numpy import generic, dtype, number, object_, bool_, _FloatValue
from numpy.typing import (
    NDArray,
    ArrayLike,
    DTypeLike,
    _ArrayLikeNumber_co,
    _ArrayLikeObject_co,
    _ArrayLikeTD64_co,
    _ArrayLikeDT64_co,
)

from unittest.case import (
    SkipTest as SkipTest,
)

_T = TypeVar("_T")
_ET = TypeVar("_ET", bound=BaseException)
_FT = TypeVar("_FT", bound=Callable[..., Any])

# Must return a bool or an ndarray/generic type
# that is supported by `np.logical_and.reduce`
_ComparisonFunc = Callable[
    [NDArray[Any], NDArray[Any]],
    Union[
        bool,
        bool_,
        number[Any],
        NDArray[Union[bool_, number[Any], object_]],
    ],
]

__all__: List[str]

class KnownFailureException(Exception): ...
class IgnoreException(Exception): ...

class clear_and_catch_warnings(warnings.catch_warnings):
    class_modules: ClassVar[Tuple[types.ModuleType, ...]]
    modules: Set[types.ModuleType]
    @overload
    def __new__(
        cls,
        record: L[False] = ...,
        modules: Iterable[types.ModuleType] = ...,
    ) -> _clear_and_catch_warnings_without_records: ...
    @overload
    def __new__(
        cls,
        record: L[True],
        modules: Iterable[types.ModuleType] = ...,
    ) -> _clear_and_catch_warnings_with_records: ...
    @overload
    def __new__(
        cls,
        record: bool,
        modules: Iterable[types.ModuleType] = ...,
    ) -> clear_and_catch_warnings: ...
    def __enter__(self) -> None | List[warnings.WarningMessage]: ...
    def __exit__(
        self,
        __exc_type: None | Type[BaseException] = ...,
        __exc_val: None | BaseException = ...,
        __exc_tb: None | types.TracebackType = ...,
    ) -> None: ...

# Type-check only `clear_and_catch_warnings` subclasses for both values of the
# `record` parameter. Copied from the stdlib `warnings` stubs.

@type_check_only
class _clear_and_catch_warnings_with_records(clear_and_catch_warnings):
    def __enter__(self) -> List[warnings.WarningMessage]: ...

@type_check_only
class _clear_and_catch_warnings_without_records(clear_and_catch_warnings):
    def __enter__(self) -> None: ...

class suppress_warnings:
    log: List[warnings.WarningMessage]
    def __init__(
        self,
        forwarding_rule: L["always", "module", "once", "location"] = ...,
    ) -> None: ...
    def filter(
        self,
        category: Type[Warning] = ...,
        message: str = ...,
        module: None | types.ModuleType = ...,
    ) -> None: ...
    def record(
        self,
        category: Type[Warning] = ...,
        message: str = ...,
        module: None | types.ModuleType = ...,
    ) -> List[warnings.WarningMessage]: ...
    def __enter__(self: _T) -> _T: ...
    def __exit__(
        self,
        __exc_type: None | Type[BaseException] = ...,
        __exc_val: None | BaseException = ...,
        __exc_tb: None | types.TracebackType = ...,
    ) -> None: ...
    def __call__(self, func: _FT) -> _FT: ...

verbose: int
IS_PYPY: Final[bool]
IS_PYSTON: Final[bool]
HAS_REFCOUNT: Final[bool]
HAS_LAPACK64: Final[bool]

def assert_(val: object, msg: str | Callable[[], str] = ...) -> None: ...

# Contrary to runtime we can't do `os.name` checks while type checking,
# only `sys.platform` checks
if sys.platform == "win32" or sys.platform == "cygwin":
    def memusage(processName: str = ..., instance: int = ...) -> int: ...
elif sys.platform == "linux":
    def memusage(_proc_pid_stat: str | bytes | os.PathLike[Any] = ...) -> None | int: ...
else:
    def memusage() -> NoReturn: ...

if sys.platform == "linux":
    def jiffies(
        _proc_pid_stat: str | bytes | os.PathLike[Any] = ...,
        _load_time: List[float] = ...,
    ) -> int: ...
else:
    def jiffies(_load_time: List[float] = ...) -> int: ...

def build_err_msg(
    arrays: Iterable[object],
    err_msg: str,
    header: str = ...,
    verbose: bool = ...,
    names: Sequence[str] = ...,
    precision: None | SupportsIndex = ...,
) -> str: ...

def assert_equal(
    actual: object,
    desired: object,
    err_msg: str = ...,
    verbose: bool = ...,
) -> None: ...

def print_assert_equal(
    test_string: str,
    actual: object,
    desired: object,
) -> None: ...

def assert_almost_equal(
    actual: _ArrayLikeNumber_co | _ArrayLikeObject_co,
    desired: _ArrayLikeNumber_co | _ArrayLikeObject_co,
    decimal: int = ...,
    err_msg: str = ...,
    verbose: bool = ...,
) -> None: ...

# Anything that can be coerced into `builtins.float`
def assert_approx_equal(
    actual: _FloatValue,
    desired: _FloatValue,
    significant: int = ...,
    err_msg: str = ...,
    verbose: bool = ...,
) -> None: ...

def assert_array_compare(
    comparison: _ComparisonFunc,
    x: ArrayLike,
    y: ArrayLike,
    err_msg: str = ...,
    verbose: bool = ...,
    header: str = ...,
    precision: SupportsIndex = ...,
    equal_nan: bool = ...,
    equal_inf: bool = ...,
) -> None: ...

def assert_array_equal(
    x: ArrayLike,
    y: ArrayLike,
    err_msg: str = ...,
    verbose: bool = ...,
) -> None: ...

def assert_array_almost_equal(
    x: _ArrayLikeNumber_co | _ArrayLikeObject_co,
    y: _ArrayLikeNumber_co | _ArrayLikeObject_co,
    decimal: float = ...,
    err_msg: str = ...,
    verbose: bool = ...,
) -> None: ...

@overload
def assert_array_less(
    x: _ArrayLikeNumber_co | _ArrayLikeObject_co,
    y: _ArrayLikeNumber_co | _ArrayLikeObject_co,
    err_msg: str = ...,
    verbose: bool = ...,
) -> None: ...
@overload
def assert_array_less(
    x: _ArrayLikeTD64_co,
    y: _ArrayLikeTD64_co,
    err_msg: str = ...,
    verbose: bool = ...,
) -> None: ...
@overload
def assert_array_less(
    x: _ArrayLikeDT64_co,
    y: _ArrayLikeDT64_co,
    err_msg: str = ...,
    verbose: bool = ...,
) -> None: ...

def runstring(
    astr: str | bytes | types.CodeType,
    dict: None | Dict[str, Any],
) -> Any: ...

def assert_string_equal(actual: str, desired: str) -> None: ...

def rundocs(
    filename: None | str | os.PathLike[str] = ...,
    raise_on_error: bool = ...,
) -> None: ...

def raises(*args: Type[BaseException]) -> Callable[[_FT], _FT]: ...

@overload
def assert_raises(  # type: ignore
    expected_exception: Type[BaseException] | Tuple[Type[BaseException], ...],
    callable: Callable[..., Any],
    /,
    *args: Any,
    **kwargs: Any,
) -> None: ...
@overload
def assert_raises(
    expected_exception: Type[_ET] | Tuple[Type[_ET], ...],
    *,
    msg: None | str = ...,
) -> unittest.case._AssertRaisesContext[_ET]: ...

@overload
def assert_raises_regex(
    expected_exception: Type[BaseException] | Tuple[Type[BaseException], ...],
    expected_regex: str | bytes | Pattern[Any],
    callable: Callable[..., Any],
    /,
    *args: Any,
    **kwargs: Any,
) -> None: ...
@overload
def assert_raises_regex(
    expected_exception: Type[_ET] | Tuple[Type[_ET], ...],
    expected_regex: str | bytes | Pattern[Any],
    *,
    msg: None | str = ...,
) -> unittest.case._AssertRaisesContext[_ET]: ...

def decorate_methods(
    cls: Type[Any],
    decorator: Callable[[Callable[..., Any]], Any],
    testmatch: None | str | bytes | Pattern[Any] = ...,
) -> None: ...

def measure(
    code_str: str | bytes | ast.mod | ast.AST,
    times: int = ...,
    label: None | str = ...,
) -> float: ...

@overload
def assert_allclose(
    actual: _ArrayLikeNumber_co | _ArrayLikeObject_co,
    desired: _ArrayLikeNumber_co | _ArrayLikeObject_co,
    rtol: float = ...,
    atol: float = ...,
    equal_nan: bool = ...,
    err_msg: str = ...,
    verbose: bool = ...,
) -> None: ...
@overload
def assert_allclose(
    actual: _ArrayLikeTD64_co,
    desired: _ArrayLikeTD64_co,
    rtol: float = ...,
    atol: float = ...,
    equal_nan: bool = ...,
    err_msg: str = ...,
    verbose: bool = ...,
) -> None: ...

def assert_array_almost_equal_nulp(
    x: _ArrayLikeNumber_co,
    y: _ArrayLikeNumber_co,
    nulp: float = ...,
) -> None: ...

def assert_array_max_ulp(
    a: _ArrayLikeNumber_co,
    b: _ArrayLikeNumber_co,
    maxulp: float = ...,
    dtype: DTypeLike = ...,
) -> NDArray[Any]: ...

@overload
def assert_warns(
    warning_class: Type[Warning],
) -> contextlib._GeneratorContextManager[None]: ...
@overload
def assert_warns(
    warning_class: Type[Warning],
    func: Callable[..., _T],
    /,
    *args: Any,
    **kwargs: Any,
) -> _T: ...

@overload
def assert_no_warnings() -> contextlib._GeneratorContextManager[None]: ...
@overload
def assert_no_warnings(
    func: Callable[..., _T],
    /,
    *args: Any,
    **kwargs: Any,
) -> _T: ...

@overload
def tempdir(
    suffix: None = ...,
    prefix: None = ...,
    dir: None = ...,
) -> contextlib._GeneratorContextManager[str]: ...
@overload
def tempdir(
    suffix: None | AnyStr = ...,
    prefix: None | AnyStr = ...,
    dir: None | AnyStr | os.PathLike[AnyStr] = ...,
) -> contextlib._GeneratorContextManager[AnyStr]: ...

@overload
def temppath(
    suffix: None = ...,
    prefix: None = ...,
    dir: None = ...,
    text: bool = ...,
) -> contextlib._GeneratorContextManager[str]: ...
@overload
def temppath(
    suffix: None | AnyStr = ...,
    prefix: None | AnyStr = ...,
    dir: None | AnyStr | os.PathLike[AnyStr] = ...,
    text: bool = ...,
) -> contextlib._GeneratorContextManager[AnyStr]: ...

@overload
def assert_no_gc_cycles() -> contextlib._GeneratorContextManager[None]: ...
@overload
def assert_no_gc_cycles(
    func: Callable[..., Any],
    /,
    *args: Any,
    **kwargs: Any,
) -> None: ...

def break_cycles() -> None: ...



from __future__ import annotations

from ._version import get_versions

__ALL__ = ['version', '__version__', 'full_version', 'git_revision', 'release']

vinfo: dict[str, str] = get_versions()
version = vinfo["version"]
__version__ = vinfo.get("closest-tag", vinfo["version"])
full_version = vinfo['version']
git_revision = vinfo['full-revisionid']
release = 'dev0' not in version and '+' not in version
short_version = vinfo['version'].split("+")[0]

del get_versions, vinfo



import numpy as np
from numpy.typing import NDArray
from typing import Any, List

i8: np.int64
f8: np.float64

AR_b: NDArray[np.bool_]
AR_i8: NDArray[np.int64]
AR_f8: NDArray[np.float64]

AR_LIKE_f8: List[float]

reveal_type(np.take_along_axis(AR_f8, AR_i8, axis=1))  # E: ndarray[Any, dtype[{float64}]]
reveal_type(np.take_along_axis(f8, AR_i8, axis=None))  # E: ndarray[Any, dtype[{float64}]]

reveal_type(np.put_along_axis(AR_f8, AR_i8, "1.0", axis=1))  # E: None

reveal_type(np.expand_dims(AR_i8, 2))  # E: ndarray[Any, dtype[{int64}]]
reveal_type(np.expand_dims(AR_LIKE_f8, 2))  # E: ndarray[Any, dtype[Any]]

reveal_type(np.column_stack([AR_i8]))  # E: ndarray[Any, dtype[{int64}]]
reveal_type(np.column_stack([AR_LIKE_f8]))  # E: ndarray[Any, dtype[Any]]

reveal_type(np.dstack([AR_i8]))  # E: ndarray[Any, dtype[{int64}]]
reveal_type(np.dstack([AR_LIKE_f8]))  # E: ndarray[Any, dtype[Any]]

reveal_type(np.row_stack([AR_i8]))  # E: ndarray[Any, dtype[{int64}]]
reveal_type(np.row_stack([AR_LIKE_f8]))  # E: ndarray[Any, dtype[Any]]

reveal_type(np.array_split(AR_i8, [3, 5, 6, 10]))  # E: list[ndarray[Any, dtype[{int64}]]]
reveal_type(np.array_split(AR_LIKE_f8, [3, 5, 6, 10]))  # E: list[ndarray[Any, dtype[Any]]]

reveal_type(np.split(AR_i8, [3, 5, 6, 10]))  # E: list[ndarray[Any, dtype[{int64}]]]
reveal_type(np.split(AR_LIKE_f8, [3, 5, 6, 10]))  # E: list[ndarray[Any, dtype[Any]]]

reveal_type(np.hsplit(AR_i8, [3, 5, 6, 10]))  # E: list[ndarray[Any, dtype[{int64}]]]
reveal_type(np.hsplit(AR_LIKE_f8, [3, 5, 6, 10]))  # E: list[ndarray[Any, dtype[Any]]]

reveal_type(np.vsplit(AR_i8, [3, 5, 6, 10]))  # E: list[ndarray[Any, dtype[{int64}]]]
reveal_type(np.vsplit(AR_LIKE_f8, [3, 5, 6, 10]))  # E: list[ndarray[Any, dtype[Any]]]

reveal_type(np.dsplit(AR_i8, [3, 5, 6, 10]))  # E: list[ndarray[Any, dtype[{int64}]]]
reveal_type(np.dsplit(AR_LIKE_f8, [3, 5, 6, 10]))  # E: list[ndarray[Any, dtype[Any]]]

reveal_type(np.lib.shape_base.get_array_prepare(AR_i8))  # E: lib.shape_base._ArrayPrepare
reveal_type(np.lib.shape_base.get_array_prepare(AR_i8, 1))  # E: Union[None, lib.shape_base._ArrayPrepare]

reveal_type(np.get_array_wrap(AR_i8))  # E: lib.shape_base._ArrayWrap
reveal_type(np.get_array_wrap(AR_i8, 1))  # E: Union[None, lib.shape_base._ArrayWrap]

reveal_type(np.kron(AR_b, AR_b))  # E: ndarray[Any, dtype[bool_]]
reveal_type(np.kron(AR_b, AR_i8))  # E: ndarray[Any, dtype[signedinteger[Any]]]
reveal_type(np.kron(AR_f8, AR_f8))  # E: ndarray[Any, dtype[floating[Any]]]

reveal_type(np.tile(AR_i8, 5))  # E: ndarray[Any, dtype[{int64}]]
reveal_type(np.tile(AR_LIKE_f8, [2, 2]))  # E: ndarray[Any, dtype[Any]]



from typing import Any
import numpy.typing as npt

AR_Any: npt.NDArray[Any]

# Mypy bug where overload ambiguity is ignored for `Any`-parametrized types;
# xref numpy/numpy#20099 and python/mypy#11347
#
# The expected output would be something akin to `ndarray[Any, dtype[Any]]`
reveal_type(AR_Any + 2)  # E: ndarray[Any, dtype[signedinteger[Any]]]



from typing import TypeVar, Union
import numpy as np
import numpy.typing as npt

T1 = TypeVar("T1", bound=npt.NBitBase)
T2 = TypeVar("T2", bound=npt.NBitBase)

def add(a: np.floating[T1], b: np.integer[T2]) -> np.floating[Union[T1, T2]]:
    return a + b

i8: np.int64
i4: np.int32
f8: np.float64
f4: np.float32

reveal_type(add(f8, i8))  # E: {float64}
reveal_type(add(f4, i8))  # E: {float64}
reveal_type(add(f8, i4))  # E: {float64}
reveal_type(add(f4, i4))  # E: {float32}



from typing import Any
import numpy as np
import numpy.typing as npt

mat: np.matrix[Any, np.dtype[np.int64]]
ar_f8: npt.NDArray[np.float64]

reveal_type(mat * 5)  # E: matrix[Any, Any]
reveal_type(5 * mat)  # E: matrix[Any, Any]
mat *= 5

reveal_type(mat**5)  # E: matrix[Any, Any]
mat **= 5

reveal_type(mat.sum())  # E: Any
reveal_type(mat.mean())  # E: Any
reveal_type(mat.std())  # E: Any
reveal_type(mat.var())  # E: Any
reveal_type(mat.prod())  # E: Any
reveal_type(mat.any())  # E: bool_
reveal_type(mat.all())  # E: bool_
reveal_type(mat.max())  # E: {int64}
reveal_type(mat.min())  # E: {int64}
reveal_type(mat.argmax())  # E: {intp}
reveal_type(mat.argmin())  # E: {intp}
reveal_type(mat.ptp())  # E: {int64}

reveal_type(mat.sum(axis=0))  # E: matrix[Any, Any]
reveal_type(mat.mean(axis=0))  # E: matrix[Any, Any]
reveal_type(mat.std(axis=0))  # E: matrix[Any, Any]
reveal_type(mat.var(axis=0))  # E: matrix[Any, Any]
reveal_type(mat.prod(axis=0))  # E: matrix[Any, Any]
reveal_type(mat.any(axis=0))  # E: matrix[Any, dtype[bool_]]
reveal_type(mat.all(axis=0))  # E: matrix[Any, dtype[bool_]]
reveal_type(mat.max(axis=0))  # E: matrix[Any, dtype[{int64}]]
reveal_type(mat.min(axis=0))  # E: matrix[Any, dtype[{int64}]]
reveal_type(mat.argmax(axis=0))  # E: matrix[Any, dtype[{intp}]]
reveal_type(mat.argmin(axis=0))  # E: matrix[Any, dtype[{intp}]]
reveal_type(mat.ptp(axis=0))  # E: matrix[Any, dtype[{int64}]]

reveal_type(mat.sum(out=ar_f8))  # E: ndarray[Any, dtype[{float64}]]
reveal_type(mat.mean(out=ar_f8))  # E: ndarray[Any, dtype[{float64}]]
reveal_type(mat.std(out=ar_f8))  # E: ndarray[Any, dtype[{float64}]]
reveal_type(mat.var(out=ar_f8))  # E: ndarray[Any, dtype[{float64}]]
reveal_type(mat.prod(out=ar_f8))  # E: ndarray[Any, dtype[{float64}]]
reveal_type(mat.any(out=ar_f8))  # E: ndarray[Any, dtype[{float64}]]
reveal_type(mat.all(out=ar_f8))  # E: ndarray[Any, dtype[{float64}]]
reveal_type(mat.max(out=ar_f8))  # E: ndarray[Any, dtype[{float64}]]
reveal_type(mat.min(out=ar_f8))  # E: ndarray[Any, dtype[{float64}]]
reveal_type(mat.argmax(out=ar_f8))  # E: ndarray[Any, dtype[{float64}]]
reveal_type(mat.argmin(out=ar_f8))  # E: ndarray[Any, dtype[{float64}]]
reveal_type(mat.ptp(out=ar_f8))  # E: ndarray[Any, dtype[{float64}]]

reveal_type(mat.T)  # E: matrix[Any, dtype[{int64}]]
reveal_type(mat.I)  # E: matrix[Any, Any]
reveal_type(mat.A)  # E: ndarray[Any, dtype[{int64}]]
reveal_type(mat.A1)  # E: ndarray[Any, dtype[{int64}]]
reveal_type(mat.H)  # E: matrix[Any, dtype[{int64}]]
reveal_type(mat.getT())  # E: matrix[Any, dtype[{int64}]]
reveal_type(mat.getI())  # E: matrix[Any, Any]
reveal_type(mat.getA())  # E: ndarray[Any, dtype[{int64}]]
reveal_type(mat.getA1())  # E: ndarray[Any, dtype[{int64}]]
reveal_type(mat.getH())  # E: matrix[Any, dtype[{int64}]]

reveal_type(np.bmat(ar_f8))  # E: matrix[Any, Any]
reveal_type(np.bmat([[0, 1, 2]]))  # E: matrix[Any, Any]
reveal_type(np.bmat("mat"))  # E: matrix[Any, Any]

reveal_type(np.asmatrix(ar_f8, dtype=np.int64))  # E: matrix[Any, Any]



"""Tests for :mod:`core.fromnumeric`."""

import numpy as np

A = np.array(True, ndmin=2, dtype=bool)
B = np.array(1.0, ndmin=2, dtype=np.float32)
A.setflags(write=False)
B.setflags(write=False)

a = np.bool_(True)
b = np.float32(1.0)
c = 1.0
d = np.array(1.0, dtype=np.float32)  # writeable

reveal_type(np.take(a, 0))  # E: Any
reveal_type(np.take(b, 0))  # E: Any
reveal_type(np.take(c, 0))  # E: Any
reveal_type(np.take(A, 0))  # E: Any
reveal_type(np.take(B, 0))  # E: Any
reveal_type(np.take(A, [0]))  # E: Any
reveal_type(np.take(B, [0]))  # E: Any

reveal_type(np.reshape(a, 1))  # E: ndarray[Any, Any]
reveal_type(np.reshape(b, 1))  # E: ndarray[Any, Any]
reveal_type(np.reshape(c, 1))  # E: ndarray[Any, Any]
reveal_type(np.reshape(A, 1))  # E: ndarray[Any, Any]
reveal_type(np.reshape(B, 1))  # E: ndarray[Any, Any]

reveal_type(np.choose(a, [True, True]))  # E: Any
reveal_type(np.choose(A, [True, True]))  # E: Any

reveal_type(np.repeat(a, 1))  # E: ndarray[Any, Any]
reveal_type(np.repeat(b, 1))  # E: ndarray[Any, Any]
reveal_type(np.repeat(c, 1))  # E: ndarray[Any, Any]
reveal_type(np.repeat(A, 1))  # E: ndarray[Any, Any]
reveal_type(np.repeat(B, 1))  # E: ndarray[Any, Any]

# TODO: Add tests for np.put()

reveal_type(np.swapaxes(A, 0, 0))  # E: ndarray[Any, Any]
reveal_type(np.swapaxes(B, 0, 0))  # E: ndarray[Any, Any]

reveal_type(np.transpose(a))  # E: ndarray[Any, Any]
reveal_type(np.transpose(b))  # E: ndarray[Any, Any]
reveal_type(np.transpose(c))  # E: ndarray[Any, Any]
reveal_type(np.transpose(A))  # E: ndarray[Any, Any]
reveal_type(np.transpose(B))  # E: ndarray[Any, Any]

reveal_type(np.partition(a, 0, axis=None))  # E: ndarray[Any, Any]
reveal_type(np.partition(b, 0, axis=None))  # E: ndarray[Any, Any]
reveal_type(np.partition(c, 0, axis=None))  # E: ndarray[Any, Any]
reveal_type(np.partition(A, 0))  # E: ndarray[Any, Any]
reveal_type(np.partition(B, 0))  # E: ndarray[Any, Any]

reveal_type(np.argpartition(a, 0))  # E: Any
reveal_type(np.argpartition(b, 0))  # E: Any
reveal_type(np.argpartition(c, 0))  # E: Any
reveal_type(np.argpartition(A, 0))  # E: Any
reveal_type(np.argpartition(B, 0))  # E: Any

reveal_type(np.sort(A, 0))  # E: ndarray[Any, Any]
reveal_type(np.sort(B, 0))  # E: ndarray[Any, Any]

reveal_type(np.argsort(A, 0))  # E: ndarray[Any, Any]
reveal_type(np.argsort(B, 0))  # E: ndarray[Any, Any]

reveal_type(np.argmax(A))  # E: {intp}
reveal_type(np.argmax(B))  # E: {intp}
reveal_type(np.argmax(A, axis=0))  # E: Any
reveal_type(np.argmax(B, axis=0))  # E: Any

reveal_type(np.argmin(A))  # E: {intp}
reveal_type(np.argmin(B))  # E: {intp}
reveal_type(np.argmin(A, axis=0))  # E: Any
reveal_type(np.argmin(B, axis=0))  # E: Any

reveal_type(np.searchsorted(A[0], 0))  # E: {intp}
reveal_type(np.searchsorted(B[0], 0))  # E: {intp}
reveal_type(np.searchsorted(A[0], [0]))  # E: ndarray[Any, Any]
reveal_type(np.searchsorted(B[0], [0]))  # E: ndarray[Any, Any]

reveal_type(np.resize(a, (5, 5)))  # E: ndarray[Any, Any]
reveal_type(np.resize(b, (5, 5)))  # E: ndarray[Any, Any]
reveal_type(np.resize(c, (5, 5)))  # E: ndarray[Any, Any]
reveal_type(np.resize(A, (5, 5)))  # E: ndarray[Any, Any]
reveal_type(np.resize(B, (5, 5)))  # E: ndarray[Any, Any]

reveal_type(np.squeeze(a))  # E: bool_
reveal_type(np.squeeze(b))  # E: {float32}
reveal_type(np.squeeze(c))  # E: ndarray[Any, Any]
reveal_type(np.squeeze(A))  # E: ndarray[Any, Any]
reveal_type(np.squeeze(B))  # E: ndarray[Any, Any]

reveal_type(np.diagonal(A))  # E: ndarray[Any, Any]
reveal_type(np.diagonal(B))  # E: ndarray[Any, Any]

reveal_type(np.trace(A))  # E: Any
reveal_type(np.trace(B))  # E: Any

reveal_type(np.ravel(a))  # E: ndarray[Any, Any]
reveal_type(np.ravel(b))  # E: ndarray[Any, Any]
reveal_type(np.ravel(c))  # E: ndarray[Any, Any]
reveal_type(np.ravel(A))  # E: ndarray[Any, Any]
reveal_type(np.ravel(B))  # E: ndarray[Any, Any]

reveal_type(np.nonzero(a))  # E: tuple[ndarray[Any, Any]]
reveal_type(np.nonzero(b))  # E: tuple[ndarray[Any, Any]]
reveal_type(np.nonzero(c))  # E: tuple[ndarray[Any, Any]]
reveal_type(np.nonzero(A))  # E: tuple[ndarray[Any, Any]]
reveal_type(np.nonzero(B))  # E: tuple[ndarray[Any, Any]]

reveal_type(np.shape(a))  # E: tuple[builtins.int]
reveal_type(np.shape(b))  # E: tuple[builtins.int]
reveal_type(np.shape(c))  # E: tuple[builtins.int]
reveal_type(np.shape(A))  # E: tuple[builtins.int]
reveal_type(np.shape(B))  # E: tuple[builtins.int]

reveal_type(np.compress([True], a))  # E: ndarray[Any, Any]
reveal_type(np.compress([True], b))  # E: ndarray[Any, Any]
reveal_type(np.compress([True], c))  # E: ndarray[Any, Any]
reveal_type(np.compress([True], A))  # E: ndarray[Any, Any]
reveal_type(np.compress([True], B))  # E: ndarray[Any, Any]

reveal_type(np.clip(a, 0, 1.0))  # E: Any
reveal_type(np.clip(b, -1, 1))  # E: Any
reveal_type(np.clip(c, 0, 1))  # E: Any
reveal_type(np.clip(A, 0, 1))  # E: Any
reveal_type(np.clip(B, 0, 1))  # E: Any

reveal_type(np.sum(a))  # E: Any
reveal_type(np.sum(b))  # E: Any
reveal_type(np.sum(c))  # E: Any
reveal_type(np.sum(A))  # E: Any
reveal_type(np.sum(B))  # E: Any
reveal_type(np.sum(A, axis=0))  # E: Any
reveal_type(np.sum(B, axis=0))  # E: Any

reveal_type(np.all(a))  # E: bool_
reveal_type(np.all(b))  # E: bool_
reveal_type(np.all(c))  # E: bool_
reveal_type(np.all(A))  # E: bool_
reveal_type(np.all(B))  # E: bool_
reveal_type(np.all(A, axis=0))  # E: Any
reveal_type(np.all(B, axis=0))  # E: Any
reveal_type(np.all(A, keepdims=True))  # E: Any
reveal_type(np.all(B, keepdims=True))  # E: Any

reveal_type(np.any(a))  # E: bool_
reveal_type(np.any(b))  # E: bool_
reveal_type(np.any(c))  # E: bool_
reveal_type(np.any(A))  # E: bool_
reveal_type(np.any(B))  # E: bool_
reveal_type(np.any(A, axis=0))  # E: Any
reveal_type(np.any(B, axis=0))  # E: Any
reveal_type(np.any(A, keepdims=True))  # E: Any
reveal_type(np.any(B, keepdims=True))  # E: Any

reveal_type(np.cumsum(a))  # E: ndarray[Any, Any]
reveal_type(np.cumsum(b))  # E: ndarray[Any, Any]
reveal_type(np.cumsum(c))  # E: ndarray[Any, Any]
reveal_type(np.cumsum(A))  # E: ndarray[Any, Any]
reveal_type(np.cumsum(B))  # E: ndarray[Any, Any]

reveal_type(np.ptp(a))  # E: Any
reveal_type(np.ptp(b))  # E: Any
reveal_type(np.ptp(c))  # E: Any
reveal_type(np.ptp(A))  # E: Any
reveal_type(np.ptp(B))  # E: Any
reveal_type(np.ptp(A, axis=0))  # E: Any
reveal_type(np.ptp(B, axis=0))  # E: Any
reveal_type(np.ptp(A, keepdims=True))  # E: Any
reveal_type(np.ptp(B, keepdims=True))  # E: Any

reveal_type(np.amax(a))  # E: Any
reveal_type(np.amax(b))  # E: Any
reveal_type(np.amax(c))  # E: Any
reveal_type(np.amax(A))  # E: Any
reveal_type(np.amax(B))  # E: Any
reveal_type(np.amax(A, axis=0))  # E: Any
reveal_type(np.amax(B, axis=0))  # E: Any
reveal_type(np.amax(A, keepdims=True))  # E: Any
reveal_type(np.amax(B, keepdims=True))  # E: Any

reveal_type(np.amin(a))  # E: Any
reveal_type(np.amin(b))  # E: Any
reveal_type(np.amin(c))  # E: Any
reveal_type(np.amin(A))  # E: Any
reveal_type(np.amin(B))  # E: Any
reveal_type(np.amin(A, axis=0))  # E: Any
reveal_type(np.amin(B, axis=0))  # E: Any
reveal_type(np.amin(A, keepdims=True))  # E: Any
reveal_type(np.amin(B, keepdims=True))  # E: Any

reveal_type(np.prod(a))  # E: Any
reveal_type(np.prod(b))  # E: Any
reveal_type(np.prod(c))  # E: Any
reveal_type(np.prod(A))  # E: Any
reveal_type(np.prod(B))  # E: Any
reveal_type(np.prod(A, axis=0))  # E: Any
reveal_type(np.prod(B, axis=0))  # E: Any
reveal_type(np.prod(A, keepdims=True))  # E: Any
reveal_type(np.prod(B, keepdims=True))  # E: Any
reveal_type(np.prod(b, out=d))  # E: Any
reveal_type(np.prod(B, out=d))  # E: Any

reveal_type(np.cumprod(a))  # E: ndarray[Any, Any]
reveal_type(np.cumprod(b))  # E: ndarray[Any, Any]
reveal_type(np.cumprod(c))  # E: ndarray[Any, Any]
reveal_type(np.cumprod(A))  # E: ndarray[Any, Any]
reveal_type(np.cumprod(B))  # E: ndarray[Any, Any]

reveal_type(np.ndim(a))  # E: int
reveal_type(np.ndim(b))  # E: int
reveal_type(np.ndim(c))  # E: int
reveal_type(np.ndim(A))  # E: int
reveal_type(np.ndim(B))  # E: int

reveal_type(np.size(a))  # E: int
reveal_type(np.size(b))  # E: int
reveal_type(np.size(c))  # E: int
reveal_type(np.size(A))  # E: int
reveal_type(np.size(B))  # E: int

reveal_type(np.around(a))  # E: Any
reveal_type(np.around(b))  # E: Any
reveal_type(np.around(c))  # E: Any
reveal_type(np.around(A))  # E: Any
reveal_type(np.around(B))  # E: Any

reveal_type(np.mean(a))  # E: Any
reveal_type(np.mean(b))  # E: Any
reveal_type(np.mean(c))  # E: Any
reveal_type(np.mean(A))  # E: Any
reveal_type(np.mean(B))  # E: Any
reveal_type(np.mean(A, axis=0))  # E: Any
reveal_type(np.mean(B, axis=0))  # E: Any
reveal_type(np.mean(A, keepdims=True))  # E: Any
reveal_type(np.mean(B, keepdims=True))  # E: Any
reveal_type(np.mean(b, out=d))  # E: Any
reveal_type(np.mean(B, out=d))  # E: Any

reveal_type(np.std(a))  # E: Any
reveal_type(np.std(b))  # E: Any
reveal_type(np.std(c))  # E: Any
reveal_type(np.std(A))  # E: Any
reveal_type(np.std(B))  # E: Any
reveal_type(np.std(A, axis=0))  # E: Any
reveal_type(np.std(B, axis=0))  # E: Any
reveal_type(np.std(A, keepdims=True))  # E: Any
reveal_type(np.std(B, keepdims=True))  # E: Any
reveal_type(np.std(b, out=d))  # E: Any
reveal_type(np.std(B, out=d))  # E: Any

reveal_type(np.var(a))  # E: Any
reveal_type(np.var(b))  # E: Any
reveal_type(np.var(c))  # E: Any
reveal_type(np.var(A))  # E: Any
reveal_type(np.var(B))  # E: Any
reveal_type(np.var(A, axis=0))  # E: Any
reveal_type(np.var(B, axis=0))  # E: Any
reveal_type(np.var(A, keepdims=True))  # E: Any
reveal_type(np.var(B, keepdims=True))  # E: Any
reveal_type(np.var(b, out=d))  # E: Any
reveal_type(np.var(B, out=d))  # E: Any
